// Copyright (c) 2022-2023 VMware Carbon Black, all rights reserved

package middleware

import (
	"gitlab.bit9.local/psc/ems/hinting/internal/config"
	"gitlab.bit9.local/psc/ems/hinting/internal/logger"
	"gitlab.bit9.local/psc/ems/hinting/internal/metrics"

	"github.com/gin-gonic/gin"
	"gitlab.bit9.local/psckit/pscrbac"
)

// RouteMiddlewareFactory -- Structure used bythe routing middleware
type RouteMiddlewareFactory struct {
	RequestLogger logger.Logger
	cfg           *config.Config
	metricsClient metrics.Service
}

//goland:noinspection GoMixedReceiverTypes
func (mwf RouteMiddlewareFactory) GetHandlersForRoute(name string, routeHandler gin.HandlerFunc) gin.HandlersChain {
	return gin.HandlersChain{routeHandler}
}

//goland:noinspection GoMixedReceiverTypes
func (mwf RouteMiddlewareFactory) GetHandlersForRBACRoute(name string, routeHandler gin.HandlerFunc, rbacPermissions ...pscrbac.Permission) gin.HandlersChain {
	return gin.HandlersChain{
		mwf.UserContextMiddleware(),
		mwf.RequestContextMiddleware(), mwf.RBACMiddleWare(rbacPermissions...), routeHandler,
	}
}

func NewMiddlewareFactory(log logger.Logger, metricsClient metrics.Service, cfg *config.Config) *RouteMiddlewareFactory {
	return &RouteMiddlewareFactory{RequestLogger: log, cfg: cfg, metricsClient: metricsClient}
}
