// Copyright (c) 2022-2023 VMware Carbon Black, all rights reserved

package repos

import (
	"context"
	"testing"
	"time"

	"gitlab.bit9.local/psc/ems/hinting/internal/app/models"
	"gitlab.bit9.local/psc/ems/hinting/internal/dbwrapper"

	"github.com/DATA-DOG/go-sqlmock"
	"gorm.io/driver/postgres"
	"gorm.io/gorm"
)

// FIXME: Unused!
//
//goland:noinspection GoUnusedExportedType
type ModelMocker[T models.Model] func(*T) []sqlmock.Argument

var now = time.Now()

func TestDeviceRepository_CreateADevice(t *testing.T) {
	db, mock, err := sqlmock.New()
	if err != nil {
		t.Fatalf("an error '%s' was not expected when opening a stub database connection", err)
	}
	//goland:noinspection GoUnhandledErrorResult
	defer db.Close()

	gormdb, err := gorm.Open(postgres.New(postgres.Config{Conn: db}))
	if err != nil {
		t.Fatalf("an error '%s' was not expected when opening a stub database connection", err)
	}
	dbClientWrapper := dbwrapper.DbClient{DB: gormdb, DbHost: "MOCKHOST"}
	mockDeviceRepository := NewRepository[*models.Device](&dbClientWrapper)
	testdevice := models.NewDeviceWindowsEndpoint(1, "orgkey", &now)

	mock.ExpectBegin()
	anyArg := sqlmock.AnyArg()
	mock.ExpectQuery(`INSERT INTO "devices"`).WithArgs(anyArg, anyArg, anyArg, anyArg, anyArg, anyArg, anyArg, anyArg, anyArg, anyArg, anyArg, anyArg, 1).WillReturnRows(sqlmock.NewRows([]string{"id"}))
	mock.ExpectCommit()

	// now we execute our method
	if err = mockDeviceRepository.Update(context.TODO(), testdevice); err != nil {
		t.Errorf("error was not expected while creating device: %s", err)
	}

	// we make sure that all expectations were met
	if err := mock.ExpectationsWereMet(); err != nil {
		t.Errorf("there were unfulfilled expectations: %s", err)
	}
}
