// Copyright (c) 2022-2023 VMware Carbon Black, all rights reserved

package repos

import (
	"context"
	"fmt"

	"gitlab.bit9.local/psc/ems/hinting/internal/app/models"
)

type MockRepository[T any, P models.ModelPointer[T]] struct {
	err   error
	model P
}

func NewMockRepository[T any, P models.ModelPointer[T]]() Repository[P] {
	return MockRepository[T, P]{err: nil}
}

func NewMockRepositoryWithModel[T any, P models.ModelPointer[T]](model P) Repository[P] {
	return MockRepository[T, P]{err: nil, model: model}
}

func NewMockRepositoryWithError[T any, P models.ModelPointer[T]]() Repository[P] {
	return MockRepository[T, P]{err: fmt.Errorf("a database error has occurred")}
}

//goland:noinspection GoUnusedParameter
func (repo MockRepository[T, P]) Get(ctx context.Context, model P) (err error) {
	if repo.model != nil {
		*model = *repo.model
	}
	return repo.err
}

//goland:noinspection GoUnusedParameter
func (repo MockRepository[T, P]) Update(ctx context.Context, model P) (err error) {
	return repo.err
}
