package redisclient

import (
	"context"
	"crypto/tls"
	"errors"
	"strings"
	"time"

	"gitlab.bit9.local/psc/ems/hinting/internal/config"

	"github.com/redis/go-redis/v9"
)

type goRedisClient struct {
	client redis.UniversalClient
	ttl    time.Duration
}

func nonRedisError(err error) error {
	if errors.Is(err, redis.Nil) {
		return nil
	}
	return err
}

func wrapGoRedisNil(err error) error {
	if errors.Is(err, redis.Nil) {
		return ErrRedisNil
	}
	return err
}

func (grc *goRedisClient) Get(ctx context.Context, cacheKey string) ([]byte, error) {
	bytes, err := grc.client.Get(ctx, cacheKey).Bytes()
	if err != nil {
		return nil, wrapGoRedisNil(err)
	}
	return bytes, err
}

func (grc *goRedisClient) Set(ctx context.Context, cacheKey string, data []byte) error {
	return nonRedisError(grc.client.Set(ctx, cacheKey, data, grc.ttl).Err())
}

func (grc *goRedisClient) Del(ctx context.Context, cacheKey string) error {
	return nonRedisError(grc.client.Unlink(ctx, cacheKey).Err())
}

func (grc *goRedisClient) Close() error {
	return grc.client.Close()
}

func (grc *goRedisClient) Ping(ctx context.Context) error {
	return grc.client.Ping(ctx).Err()
}

func newGoRedisClient(cfg *config.Config) (RedisClient, error) {
	rdb := redis.NewClusterClient(&redis.ClusterOptions{
		Addrs:         strings.Split(cfg.NoSQLConnection, ","),
		ReadOnly:      true,
		RouteRandomly: true,
		WriteTimeout:  cfg.NoSQLWriteTimeout,
		ReadTimeout:   cfg.NoSQLReadTimeout,
		PoolTimeout:   cfg.NoSQLPoolTimeout,
		TLSConfig:     &tls.Config{MinVersion: tls.VersionTLS12},
		Password:      cfg.NoSQLAuthToken,
	})
	return &goRedisClient{client: rdb, ttl: cfg.NoSQLTTL}, nil
}
